# coding=utf-8
import inspect
from textwrap import dedent
import re
from nekbot.utils.iter import append_or_update
from nekbot.utils.survey import InspectFunction

__author__ = 'nekmo'


class Doc(InspectFunction):
    msg = 'Usage for command {name}: {command} {args_doc}'
    msg_base  = 'Usage: {command} {args_doc}'

    def __init__(self, name, command):
        super(Doc, self).__init__()
        self.name = name
        self.command = command
        self.arg_names = []

    def process_doc_msg(self, text):
        return re.sub(' +', ' ', dedent(text).replace('\n', ' '))

    def set_from_function(self, function):
        super(Doc, self).set_from_function(function)
        self.arg_names = inspect.getargspec(function).args[1:]  # Ignoro el primero, que es el Msg
        self.kwargs_values = inspect.getargspec(function).defaults
        self.kwargs_names = inspect.getargspec(function).args[-1 * len(self.kwargs_values if self.kwargs_values else
                                                                       []):]
        if function.__doc__:
            self.msg = self.process_doc_msg(function.__doc__)

    def args_doc(self):
        args = [('<%s (%s)>' % (self.arg_names[i], self.get_type_name(type))) for i, type
                in enumerate(self.arg_types)]
        if self.kwargs_values:
            args += ['[%s (%s)]' % (name, self.kwargs_values[i]) for i, name
                     in enumerate(self.kwargs_names)]
        return ' '.join(args)

    def get_usage(self):
        return '%s %s' % (self.command, self.args_doc())

    def __str__(self):
        return self.msg.format(**{
            'name': self.name, 'command': self.command, 'msg_base': self.msg_base,
            'args_doc': self.args_doc(), 'usage': self.get_usage(),
        })


# coding=utf-8
import shlex
import re
from nekbot.utils.ints import get_int

__author__ = 'nekmo'



def long_message(message, newlines=1, length=140):
    if not isinstance(message, (str, unicode)):
        message = str(message)
    if len(message) > length:
        return True
    if len(message.split('\n')) - 1 > newlines:
        return True
    return False


def split_arguments(body):
    """Dividir una cadena de texto en un listado de argumentos como en un shell.
    """
    # Sustituyo todas las comillas sencillas entre palabras por car \x00 para luego
    # devolverlo a su original. Hago esto, porque seguramente sea un apóstrofe como
    # los usados en inglés.
    body = re.sub(r"([^\A ])\'([^\Z ])", "\\1\x00\\2", body)
    args = shlex.split(body)
    args = map(lambda x: x.replace('\x00', "'"), args)
    return args


def find_occurrences(key, text):
    return map(lambda x: x.start(), re.finditer(re.escape(key), text))


def limit_context(key, text, chars_context=10, limiter='[...]'):
    results = []
    for occurrence in find_occurrences(key, text):
        part_a = text[get_int(occurrence - chars_context, 0):occurrence]
        part_b = text[occurrence + len(key):get_int(occurrence + len(key) + chars_context, len(text))]
        results.append('{limiter}%s{key}%s{limiter}'.format(**locals()) % (part_a, part_b))
    return ' '.join(results)


def highlight_occurrence(text, occurrence, char='*'):
    return re.sub('(%s)' % re.escape(occurrence), r'{char}\1{char}'.format(char=char), text)


def in_str_no_case(term, text):
    return bool(re.findall(re.escape(term), text, re.IGNORECASE))


def replaces(text, **kwargs):
    for key, value in kwargs.items():
        text = text.replace(key, value)
    return text


# coding=utf-8
from nekbot.core.exceptions import PrintableException
from nekbot.protocols import Message
from nekbot.utils.strings import split_arguments

__author__ = 'nekmo'

import pytest
from nekbot.core.types.argparse import Text


def test_text_all_message():
    """Comprobar si funciona el tipo Text siendo este todo el mensaje
    """
    msg = Message('test', 'spam spam spam', 'test')
    text = Text()
    assert text.parse('spam', ['spam', 'spam'], msg, 1) == 'spam spam spam'


def test_text_second_argument():
    """Probando el tipo text, siendo este el segundo argumento
    """
    msg = Message('test', 'spam spam spam', 'test')
    text = Text()
    assert text.parse('spam', ['spam'], msg, 2) == 'spam spam'


def test_text_min_length():
    """Probando tipo text con mín. de caracteres no satisfecho
    """
    msg = Message('test', 'spam spam spam', 'test')
    text = Text(30)
    with pytest.raises(PrintableException):
        text.parse('spam', ['spam'], msg, 2)


def test_text_max_length():
    """Probar el tipo text con max. de caracteres
    """
    msg = Message('test', 'spam spam spam', 'test')
    text = Text(max_length=6)
    args = ['spam']
    assert text.parse('spam', args, msg, 2) == 'spam s'
    assert args == ['pam']


def test_text_quote():
    """Debería devolverse el texto entero, sin fallar por la comilla
    """
    body = 'nobody\'s specs the spanish inquisition'
    msg = Message('test', body, 'test')
    text = Text()
    args = split_arguments(msg.body)
    assert text.parse(args[0], args[1:], msg, 1) == body


def test_text_no_args():
    """Tras tomarse el texto entero, no deberían quedar argumentos
    """
    body = 'nobody\'s specs the spanish inquisition'
    msg = Message('test', body, 'test')
    text = Text()
    split_args = split_arguments(msg.body)
    args = split_args[1:]
    text.parse(split_args[0], args, msg, 1)
    assert args == []


def test_text_quotes():
    """Debería devolverse la frase original con los quotes
    """
    body = 'spam "spam eggs" spam'
    msg = Message('test', body, 'test')
    text = Text()
    args = split_arguments(msg.body)
    assert text.parse(args[0], args[1:], msg, 1) == body



from nekbot.core.commands import cmds, command
from nekbot.core.types.argparse import Text, SetBool

__author__ = 'nekmo'

@command('commands', search=Text, by_plugin=SetBool('-p'))
def commands(msg, search='', by_plugin=False):
    if search:
        response = ''
        results_cmd, results_doc = cmds.search(search)
        if results_cmd:
            response += '- Results in commands names: %s' % ', '.join(sorted(results_cmd))
        if results_cmd and results_doc:
            response += '\n'
        if results_doc:
            response += '- Results in documentation: %s' % ', '.join(results_doc)
        return response
    return 'Available commands: %s. Tip!! Use <command> -h for help.' % ' '.join(sorted(cmds.keys()))




# coding=utf-8
import datetime
from nekbot import _

__author__ = 'nekmo'

# TIME_TIMES = {'d': 3600 * 24, 'h': 3600, 'm': 60, 's': 1}
# HUMAN_TIMES = {
# 's': (_('second'), _('seconds')),
# 'm': (_('minute'), _('minutes')),
# 'h': (_('hour'), _('hours')),
# 'd': (_('day'), _('days')),
# }

MAGNITUDES = {
    'days': (_("day"), _("days")),
    'hours': (_("hour"), _("hours")),
    'minutes': (_("minute"), _("minutes")),
    'seconds': (_("second"), _("seconds")),
}


def human_join(list, and_='and', sep=', '):
    if len(list) == 1:
        return list[0]
    return '%s %s %s' % (sep.join(list[:-1]), and_, list[-1])


def strdelta(delta, join_str=', '):
    days, rem = divmod(delta.total_seconds(), 86400)
    hours, rem = divmod(rem, 3600)
    minutes, seconds = divmod(rem, 60)
    if seconds < 1: seconds = 1
    locals_ = locals()
    magnitudes_str = []
    for magnitude in ('days', 'hours', 'minutes', 'seconds'):
        value = locals_[magnitude]
        if not value:
            continue
        magnitudes_str.append('%i %s' % (value, MAGNITUDES[magnitude][1] if value > 1
                                                else MAGNITUDES[magnitude][1]))
    eta_str = join_str.join(magnitudes_str)
    return eta_str


import os
from nekbot import settings

__author__ = 'nekmo'

import pyejdb

def ejdb(plugin, collection='global'):
    name = 'ejdb-%s' % collection
    path = os.path.join(settings.STORAGE_DIR, plugin, name)
    try:
        os.makedirs(os.path.dirname(path))
    except:
        pass
    return pyejdb.EJDB(path, pyejdb.DEFAULT_OPEN_MODE | pyejdb.JBOTRUNC)


from nekbot.utils.strings import split_arguments

__author__ = 'nekmo'


def test_split_args_quote():
    """.
    """
    assert split_arguments('nobody\'s specs the spanish inquisition') == ['nobody\'s', 'specs', 'the',
                                                                          'spanish', 'inquisition']


from meuhdb import MeuhDb

__author__ = 'nekmo'

class Meuh(object):
    def __init__(self, plugin, name='global'):
        self.db_name = '%s-%s.json' % (plugin, name)
        self.meuh = MeuhDb(
            path=None,
            autocommit=False, autocommit_after=3,
            lazy_indexes=False)


# coding=utf-8
from inspect import getargspec
from nekbot.utils.iter import append_or_update


def kwargs_function(function):
    argspec = getargspec(function)
    kwarg_names = argspec.args[-len(argspec.defaults):]
    return {kwarg_name: argspec.defaults[i] for i, kwarg_name in enumerate(kwarg_names)}


class InspectFunction(object):
    def __init__(self):
        self.arg_types = []
        self.kwarg_types = []

    def set_arg_types(self, arg_types):
        append_or_update(self.arg_types, arg_types)

    def set_kwarg_types(self, kwarg_types):
        append_or_update(self.kwarg_types, kwarg_types)
    
    def set_from_function(self, function):
        argspec = getargspec(function)
        arg_types, kwargs_types = argspec.args, argspec.defaults
        if kwargs_types:
            append_or_update(self.kwarg_types, map(self.get_type, kwargs_types), False)
        # añado el argumento si no hay para la posición, pero si no no lo modifico
        # Le quito 1 porque el primer argumento es "msg", el objeto Msg
        arg_types = arg_types[:-len(kwargs_types if kwargs_types else [])]
        append_or_update(self.arg_types, [str] * (len(arg_types) - 1), False)

    def get_type(self, value):
        if hasattr(value, '__call__'):
            return value
        elif hasattr(value.__class__, '__call__'):
            return value.__class__
        else:
            return str

    def get_type_name(self, type):
        try:
            return type.__name__
        except Exception:
            return type.__class__.__name__


# coding=utf-8
from collections import defaultdict
from logging import getLogger
import traceback
import threading
import re

from nekbot import settings
from nekbot.core.commands.argparse import ArgParse
from nekbot.core.commands.doc import Doc
from nekbot.core.exceptions import PrintableException
from nekbot.utils.decorators import optional_args
from nekbot.utils.strings import split_arguments, highlight_occurrence, limit_context, in_str_no_case


__author__ = 'nekmo'

logger = getLogger('nekbot.core.commands')


class Command(object):
    symbol = True
    _doc = None

    def __init__(self, name=None, function=None, symbol=None, *args, **kwargs):
        self.name = name
        self.function = function
        self.symbol = symbol if symbol is not None else self.symbol
        self.argparse = self.get_argparse(args, kwargs, function)
        self._args = args
        self._kwargs = kwargs # TODO el orden es importante

    def get_argparse(self, arg_types, kwargs, function):
        argparse = ArgParse()
        argparse.set_arg_types(arg_types)
        argparse.set_from_function(function)
        argparse.set_kwargs(kwargs, function)
        # argparse.get_from_function(self.function)
        return argparse

    def get_doc(self):
        doc = Doc(self.name, repr(self))
        doc.set_arg_types(self._args)
        doc.set_from_function(self.function)
        self._doc = doc
        return doc

    @property
    def doc(self):
        if self._doc is not None: return self._doc
        self._doc = str(self.get_doc())
        return self._doc

    def execute(self, msg):
        if not hasattr(msg, 'args'):
            msg.args = split_arguments(msg.body)[1:]
        if '--help' in msg.args or '-h' in msg.args:
            # Return documentation.
            msg.reply(str(self.get_doc()))
            return
        try:
            args = self.argparse.parse(msg.args, msg)
        except Exception as e:
            return msg.short_reply(e)
        try:
            self.control(msg)
        except PrintableException as e:
            return msg.user.send_warning(e)
        try:
            response = self.function(msg, *args)
        except PrintableException as e:
            response = str(e)
        except Exception:
            logger.error(traceback.format_exc())
            msg.user.send_warning('El comando %s no finalizó correctamente.' % repr(self))
            return
        if response is not None:
            msg.reply(response)

    def control(self, msg):
        if hasattr(self.function, 'control'):
             return self.function.control.check(msg)
        if hasattr(self.function, 'command_decorator') and \
                hasattr(self.function.command_decorator, 'control'):
            return self.function.command_decorator.control.check(msg)
        return True

    def __repr__(self):
        if self.symbol:
            return settings.SYMBOL + self.name
        else:
            return self.name


class Commands(defaultdict):
    def __init__(self):
        super(Commands, self).__init__(list)

    def incoming(self, msg):
        if msg.is_from_me:
            return
        if not msg.body.startswith(settings.SYMBOL):
            return
        if msg.historical:
            return
        args = split_arguments(msg.body)
        if not args[0] in self:
            # No es un comando, se ignora
            return
        cmd, args = args[0], args[1:]
        msg.args = args
        for command in self[cmd]:
            l = threading.Thread(target=command.execute, args=(msg,))
            l.start()

    def add_command(self, name, function, *args, **kwargs):
        cmd = Command(name, function, *args, **kwargs)
        self[repr(cmd)].append(cmd)

    def search(self, term):
        results_cmd = []
        results_doc = []
        for cmd_repr in self.keys():
            if not in_str_no_case(term, cmd_repr):
                continue
            results_cmd.append("%s (%s)" % (cmd_repr, highlight_occurrence(cmd_repr, term)))
        for command_list in self.values():
            for command in command_list:
                if not in_str_no_case(term, command.doc):
                    continue
                context = limit_context(term, command.doc)
                results_doc.append("%s (%s)" % (
                    repr(command), highlight_occurrence(context, term)
                ))
        return results_cmd, results_doc


cmds = Commands()


@optional_args
def command(func, *args, **kwargs):
    return func(*args, **kwargs)


@optional_args
class command:
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        if len(args) > 1:
            name = args[1]
        else:
            name = args[0].func_name
        args[0].command_decorator = self
        self.name = name
        cmds.add_command(name, args[0], *args[1:], **kwargs)

    def __repr__(self):
        return '<@Command %s>' % self.name

    def __call__(self, func):
        return func(*self.args, **self.kwargs)


__author__ = 'nekmo'

def append_or_update(original_list, new_list, override=True):
    for i, arg in enumerate(new_list):
        if i < len(original_list) and override:
            original_list[i] = arg
        elif i >= len(original_list):
            original_list.append(arg)
    return original_list


import os
import shutil
import re
from nekbot.utils.strings import replaces

__author__ = 'nekmo'


class RestartRename(Exception):
    pass


def copytree(src, dst, symlinks=False, ignore=None):
    for item in os.listdir(src):
        s = os.path.join(src, item)
        d = os.path.join(dst, item)
        if os.path.isdir(s):
            shutil.copytree(s, d, symlinks, ignore)
        else:
            shutil.copy2(s, d)


def replace_inside_file(name, **kwargs):
    data = open(name, 'rb').read()
    with open(name, 'wb') as f:
        for key, value in kwargs.items():
            data = re.sub(re.escape(key), value, data)
        f.write(data)


def all_nodes(directory, file_handler=None, dir_handler=None, file_result=False, dir_result=False):
    results = {}
    for root, dirs, files in os.walk(directory):
        if file_handler is not None:
            for name in files:
                filepath = os.path.join(root, name)
                result = file_handler(filepath)
                if file_result:
                    results[filepath] = result
        if dir_handler is not None:
            for name in files:
                dirpath = os.path.join(root, name)
                result = dir_handler(dirpath)
                if dir_result:
                    results[dirpath] = result
    return results


def all_files(directory, handler, file_result=False):
    return all_nodes(directory, handler, file_result=file_result)


def all_directories(directory, handler, dir_result=False):
    return all_nodes(directory, dir_handler=handler, dir_result=dir_result)


def rename(node, new_name):
    dirname, name = os.path.split(node)
    shutil.move(node, os.path.join(dirname, new_name))


def copy_template(src, dst, to_replace=None):
    if not to_replace:
        to_replace = {}

    def rename_directory(directory):
        rename(directory, replaces(os.path.split(directory)[1], **to_replace))
        raise RestartRename
    copytree(src, dst)
    all_files(dst, lambda x: replace_inside_file(x, **to_replace))
    all_files(dst, lambda x: rename(x, replaces(os.path.split(x)[1], **to_replace)))
    while True:
        try:
            all_directories(dst, rename_directory)
        except RestartRename:
            pass
        else:
            break


# coding=utf-8
from nekbot.core.exceptions import PrintableException
from nekbot.core.types.argparse import Text
from nekbot.protocols import Message

__author__ = 'nekmo'

import pytest
from nekbot.core.commands import argparse

def test_without_args():
    """Sin dar argumentos, no debe devolver, ni debe pedir.
    """
    def myfunction(msg):
        pass
    parser = argparse.ArgParse()
    parser.set_from_function(myfunction)
    assert parser.parse([]) == []


def test_arg():
    """Debe devolver el argumento entregado.
    """
    def myfunction(msg, word):
        pass
    parser = argparse.ArgParse()
    parser.set_from_function(myfunction)
    assert parser.parse(['word']) == ['word']


def test_int_arg():
    """Debe devolver el argumento como un int.
    """
    def myfunction(msg, number):
        pass
    parser = argparse.ArgParse()
    parser.set_arg_types([int])
    parser.set_from_function(myfunction)
    assert parser.parse(['1']) == [1]


def test_not_enought_arguments():
    """Si no hay suficientes argumentos, debe fallar.
    """
    def myfunction(msg, word1, word2):
        pass
    parser = argparse.ArgParse()
    parser.set_arg_types([int])
    parser.set_from_function(myfunction)
    with pytest.raises(PrintableException):
        parser.parse(['arg'])


def test_multiple_args():
    """Debe ser capaz de capturar todos los argumentos con *args
    """
    def myfunction(*args):
        pass
    parser = argparse.ArgParse()
    parser.set_from_function(myfunction)
    parser.parse(['spam', 'eggs', 'spam']) == ['spam', 'eggs', 'spam']


def test_detect_kwarg_type():
    """Debe transformar el kwarg opcional en un número
    """
    def myfunction(number=5):
        pass
    parser = argparse.ArgParse()
    parser.set_from_function(myfunction)
    parser.parse(['6']) == [6]


def test_not_require_kwargs():
    """Los kwargs no son obligatorios
    """
    def myfunction(number=5):
        pass
    parser = argparse.ArgParse()
    parser.set_from_function(myfunction)
    parser.parse([]) == []


def test_text_argument():
    """Debería devolverse un único argumento con todo el texto
    """
    def myfunction(text):
        pass
    text = 'nobody\'s specs the spanish inquisition'
    msg = Message('test', text, 'test')
    parser = argparse.ArgParse()
    parser.set_arg_types([Text()])
    parser.set_from_function(myfunction)
    assert parser.parse(text.split(' '), msg) == [text]



# coding=utf-8
import inspect
from nekbot.core.exceptions import InvalidArgument, PrintableException
from nekbot.core.types.argparse import ArgParseType
from nekbot.utils.iter import append_or_update
from nekbot.utils.survey import InspectFunction, kwargs_function

__author__ = 'nekmo'


class AllTypes(object):
    pass


ERRORS = {
    ValueError: {
        int: 'El valor debe ser un número.',
        AllTypes: 'Debe ser de tipo {type}',
    },
}


class ArgParse(InspectFunction):
    def __init__(self):
        super(ArgParse, self).__init__()
        self.special_kwargs = [] # Kwargs que pueden encontrarse en cualquier parte del boy
        self.kwarg_values = []
        self.kwags = {} # {nombre: valor por defecto en función}

    def parse_arg(self, type, value, pos=None):
        try:
            if hasattr(type, '__call__'):
                value = type(value)
        except Exception as e:
            if isinstance(e, InvalidArgument):
                # ¡Es una excepción creada por nosotros. Le damos por si lo necesitase
                # información adicional y lo lanzamos tal cual
                e.give_info(value, pos)
                raise e
            err_class = e.__class__
            # No es un tipo de excepción conocida.
            if not ERRORS.get(err_class): raise e
            # Es conocida, pero no tenemos tipo para el mismo, y no hay por defecto
            if not ERRORS[err_class].get(type) and not ERRORS[err_class].get(AllTypes):
                raise e
            # No tenemos tipo para él, pero sí por defecto
            if not ERRORS[err_class].get(type) and ERRORS[err_class].get(AllTypes):
                raise InvalidArgument(ERRORS[err_class][AllTypes], value, pos)
            # ¡Hemos triunfado! Hay para excepción->tipo
            raise InvalidArgument(ERRORS[err_class][type], value, pos)
        return value

    def set_kwargs(self, kwargs, function):
        """Esta función (a la cual se le entrega un diccionario de kwargs) debe ser utilizada
        después de set_from_function
        """
        argspec = inspect.getargspec(function)
        if not argspec.defaults:
            return # No hay kwargs
        kwarg_names = argspec.args[-len(argspec.defaults):]
        self.kwarg_values = argspec.defaults
        for key, arg_type in kwargs.items():
            try:
                index = kwarg_names.index(key)
            except ValueError:
                continue
            if index > len(self.kwarg_types):
                continue
            if inspect.isclass(arg_type) and issubclass(arg_type, ArgParseType):
                # Es la clase sin instanciar de ArgParseType. Se instancia
                arg_type = arg_type()
            if isinstance(arg_type, ArgParseType):
                arg_type.set_name(key)
                # Si el index es la posición de este elemento dentro de los kwargs, el position
                # es la posición del mismo juntando args y kwargs. Esto sirve para luego a la
                # hora de construir el listado con los argumentos a entregar a la función, el
                # kwarg especial esté en la posición correcta.
                # Hago -1 por el primer argumento (msg)
                arg_type.position = index + (len(argspec.args) - len(argspec.defaults) - 1)
                self.special_kwargs.append(arg_type)
            self.kwarg_types[index] = arg_type
        self.kwargs = kwargs_function(function)

    def parse(self, args, msg=None):
        if len(self.arg_types) > len(args):
            raise PrintableException('Not enough arguments for this command. Missing %i argument%s.' %
                                     (len(self.arg_types), 's' if len(self.arg_types) > 1 else ''))
        args = list(args)  # Listado de argumentos que me han entregado
        final_args = []
        final_kwargs = {}
        all_types = list(self.arg_types) + list(self.kwarg_types)  # Todos los tipos conocidos
        i = 0  # Argumento posicional
        while True:
            if not args:
                break
            word_now = args.pop(0)  # Tomo el elemento que me toca ahora
            type_now = None
            # Los argumentos especiales son kwargs (normalmente con forma --arg), que pueden aparecer
            # en cualquier parte del mensaje. Se mira si la palabra actual es uno de estos.
            for special_kwarg in self.special_kwargs:
                if not special_kwarg.match(word_now, args, msg, i, self):
                    continue
                type_now = special_kwarg
                final_kwargs[type_now.position] = (type_now.parse(word_now, args, msg, i, self))
                break
            if type_now is not None:
                # Ya se ha obtenido este argumento por un argumento kwarg especial
                continue
            if all_types:
                type_now = all_types.pop(0)  # Tomo el tipo que me toca ahora
            else:
                type_now = str  # No hay más tipos disponibles. Uso str por defecto
            if inspect.isclass(type_now) and issubclass(type_now, ArgParseType):
                # Es la clase sin instanciar de ArgParseType. Se instancia
                type_now = type_now()
            if not isinstance(type_now, ArgParseType):
                arg = self.parse_arg(type_now, word_now, i)
            else:
                arg = type_now.parse(word_now, args, msg, i, self)
            final_args.append(arg)
            i += 1
        # En el final args, pondré también los valores por defecto de los kwargs, para poder
        # sobrescribirlos con los valores que pudiesen haberse establecido por Special types,
        # en la posición que necesitan
        final_args += self.kwarg_values[-(len(self.arg_types) + len(self.kwarg_values) - len(final_args)):]
        for position, value in final_kwargs.items():
            final_args[position] = value
        return final_args


# coding=utf-8
import re
from nekbot.core.exceptions import InvalidArgument
from nekbot.utils.strings import split_arguments

__author__ = 'nekmo'

class ArgParseType(object):
    name = None

    def parse(self, arg, args, msg, position, parser=None):
        return arg

    def set_name(self, name):
        pass

    def match(self, arg, args, msg, position, parser):
        return False

class Text(ArgParseType):
    def __init__(self, min_length=1, max_length=None):
        self.max_length = max_length
        self.min_length = min_length

    def parse(self, arg, args, msg, position, parser=None):
        # Cojo el mensaje original, incluyendo comillas, con los argumentos que quedan
        # hasta el momento
        args.insert(0, arg) # Me han dado como primer argumento parte de la frase. La reintroduzco
        new_args = map(lambda x: '["\']?%s["\']?' % x, args)
        last_body = re.match(".*" + "(%s)$" % ' '.join(new_args), msg.body)
        if not last_body and self.min_length:
            raise InvalidArgument('The argument of type text is required.', last_body, position)
        last_body = last_body.group(1)
        if self.min_length > len(last_body):
            raise InvalidArgument('The text argument type must have at least %i characters.' % self.min_length,
                                  last_body, position)
        if self.max_length is not None:
            try:
                args[:] = split_arguments(last_body[self.max_length:])
            except ValueError:
                raise InvalidArgument('Failed to Divide up the argument of type text.'
                                      'Surely there are unmatched quotes.', last_body[self.max_length:], position)
            last_body = last_body[:self.max_length]
        else:
            # Al no haber límite de caracteres, se toma el resto de la frase. No quedan argumentos.
            args[:] = []
        return last_body

class List(ArgParseType):
    pass

class SetBool(ArgParseType):
    shortcut = ''
    argument = ''
    position = None
    value = None

    def __init__(self, *args):
        if not args:
            return
        i = 0
        for arg_type in [self.set_shortcut, self.set_argument, self.set_value]:
            if i >= len(args):
                break
            try:
                arg_type(args[i])
            except ValueError:
                pass
            finally:
                i += 1

    def set_shortcut(self, shortcut):
        if not isinstance(shortcut, (str, unicode)):
            raise ValueError
        if not re.match('^\-[A-z0-9]', shortcut):
            raise ValueError
        self.shortcut = shortcut

    def set_argument(self, argument):
        if not isinstance(argument, (str, unicode)):
            raise ValueError
        if not argument.startswith('--'):
            raise ValueError
        self.argument = argument

    def set_value(self, value):
        self.value = value

    def set_name(self, name):
        if not self.shortcut:
            self.set_shortcut('-%s' % name[0])
        if not self.argument:
            self.set_argument('--%s' % name.replace('_', '-'))
        self.name = name

    def match(self, arg, args, msg, position, parser):
        if arg not in [self.argument, self.shortcut]:
            return False
        return True

    def get_value(self, kwargs):
        """Obtener el valor a devolver con setBool
        :param kwargs: El kwargs {kwarg: valor} de la función
        :return:
        """
        if self.value is not None:
            return self.value
        value = kwargs[self.name]
        if value in [True, False]:
            return not value

    def parse(self, arg, args, msg, position, parser=None):
        value = self.get_value(parser.kwargs)
        return value


#!/usr/bin/env/python
# coding=utf-8
import os
import gettext

MODULE = 'nekbot'
LOCALES_DIR = 'locales'
__version__ = 'Mirai 0.1'
__author__ = 'nekmo'
__dir__ = os.path.abspath(os.path.dirname(__file__))

gettext.bindtextdomain(MODULE) #  os.path.join(__dir__, LOCALES_DIR)
gettext.textdomain(MODULE)
_ = gettext.gettext
_n = gettext.ngettext

from nekbot.conf import settings
from nekbot.core import NekBot
from nekbot.protocols import Protocols

t = gettext.install('nekbot', LOCALES_DIR)


if '__path__' in globals():
    from pkgutil import extend_path
    __path__ = extend_path(__path__, __name__)


# coding=utf-8
from types import IntType
from nekbot.core.commands import command
from nekbot.core.commands.control import control
from nekbot.core.commands.temp import TempRegex
from nekbot.core.exceptions import PrintableException

__author__ = 'nekmo'

@command('argnumber', int)
def argnumber(msg, number):
    assert type(number) is IntType
    msg.reply('Entregado: %i' % number)

@command
def args(msg, *args):
    msg.reply('Entregado: %s' % ', '.join(args))

@command
def kwargnumber(msg, number=10):
    assert type(number) is IntType
    return 'Entregado: %i' % number

@command
def privatemsg(msg):
    msg.user.send_message('Un mensaje privado')

@command
def publicmsg(msg):
    msg.groupchat.send_message('Un mensaje público')

@command
def test_temp_regex(msg):
    msg.reply('Elija: [Sn]')
    temp = TempRegex(msg.protocol, '^(S|s|N|n)$', timeout=5)
    response = temp.read().next()
    msg.reply('Usted ha dicho: %s' % response.match[0])

@command
@control('root')
def need_root(msg):
    return 'Hola jefe!'

@command
@control('spam')
def need_spam(msg):
    return 'Spam! Spam! Spam!!'

@command
def raise_printable_exception(msg):
    raise PrintableException("nobody's specs the spanish inquisition!")

@command
def raise_exception(msg):
    raise Exception


@command
def multiline(self):
    return """Texto
    de
    ejemplo"""

@command
def doc(msg):
    """Esto es un ejemplo de documentación.
    """
    pass

@command('docargs')
def docargs(msg, arg1):
    """Otro ejemplo de doc. pero con 1 arg. Uso: {command} {args_doc}
    """
    pass

@command('docargs2', str, int)
def docargs2(msg, arg1, arg2):
    """Otro ejemplo de doc. pero con 2  args. Uso: {usage}
    """
    pass

@command('docargs3', str, int)
def docargs3(msg, arg1, arg2, arg3=5):
    """Otro ejemplo de doc. pero con 2  args. Uso: {usage}
    """
    pass

@command
def breakline(msg):
    return 'Line1\nLine2'

@command
def get_users(msg):
    return msg.groupchat.get_users()